/* $Id: ast_exceptioncompound.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_exceptioncompound.h"
#include "ast_statement.h"
#include "ast_finally.h"
#include "ast_ehandlers.h"
#include "../astvisitor.h"

wtast::ExceptionCompound::~ExceptionCompound()
{
	while(!mStatements.empty())
	{
		delete mStatements.front();
		mStatements.pop_front();
	}

	delete mEHandlers;
	delete mFinally;
}

void wtast::ExceptionCompound::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		for(statements_t::iterator it=mStatements.begin(),end=mStatements.end();it!=end;++it)
		{
			(*it)->Accept(vis);
		}

		if(mEHandlers)
		{
			mEHandlers->Accept(vis);
		}

		if(mFinally)
		{
			mFinally->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

wtast::ExceptionCompound* wtast::ExceptionCompound::Copy()
{
	statements_t stmts;

	for(statements_t::iterator it=mStatements.begin(),end=mStatements.end();it!=end;++it)
	{
		stmts.push_back((*it)->Copy());
	}

	return new ExceptionCompound(stmts.begin(), stmts.end(), mEHandlers->Copy(), mFinally->Copy());
}

