/* $Id: ast_switchstatement.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_SWITCHSTATEMENT_H
#define WTPARSER_AST_SWITCHSTATEMENT_H

#include "../astnode.h"
#include "ast_statement.h"
#include <deque>

namespace wtast
{

class Expression;
class SwitchCase;

class SwitchStatement : public Statement
{
	typedef std::deque<SwitchCase*> cases_t;

	Expression* mCondition;
	cases_t     mCases;

public:
	SwitchStatement(Expression* cond)
		: Statement()
		, mCondition(cond)
	{
		// ok
	}

	~SwitchStatement();

	void Accept(AstVisitor* vis);

	SwitchStatement* Copy();

	Kind GetKind() const
	{
		return SWITCHSTATEMENT;
	}

	Expression* GetCondition()
	{
		return mCondition;
	}

	template<typename Iter>
	void AddCases(Iter start, Iter end)
	{
		for(;start!=end;++start)
		{
			AddCase(*start);
		}
	}

	void AddCase(SwitchCase* cas)
	{
		mCases.push_back(cas);
	}

	std::size_t GetNumberOfCases() const
	{
		return mCases.size();
	}

	SwitchCase* GetCase(std::size_t idx)
	{
		if(idx<mCases.size())
		{
			return mCases[idx];
		}
		else
		{
			return 0;
		}
	}
};

} // namespace wtast

#endif // WTPARSER_AST_SWITCHSTATEMENT_H
