#include "config_private.h"
#include "convert.h"
#include <stdlib.h>
#if WTRUNTIME_ENABLE_FLOAT32 || WTRUNTIME_ENABLE_FLOAT64
#  include <stdio.h>
#endif /* WTRUNTIME_ENABLE_FLOAT32 || WTRUNTIME_ENABLE_FLOAT64 */

#define MOD10(_m_,_i_)          (_m_) = ((_i_) % 10)
#define DIV10(_i_)              (_i_) = ((_i_) / 10)
#define MUL10(_i_)              (_i_) = (((_i_)<<3) + ((_i_)<<1))
#define ADD_MUL10(_n_,_i_)      MUL10(_i_); (_i_) += (_n_)

/*================================================================< i32 >==*/

wt_i32 wtr_convert_dec_to_i32(const char* str)
{
	wt_i32  value    = 0;
	wt_bool negative = WT_FALSE;

	if('-'==str[0])
	{
		negative = WT_TRUE;
		++str;
	}

	while(*str)
	{
		char ch = *str++;

		switch(ch)
		{
		case '0': MUL10(value);       break;
		case '1': ADD_MUL10(1,value); break;
		case '2': ADD_MUL10(2,value); break;
		case '3': ADD_MUL10(3,value); break;
		case '4': ADD_MUL10(4,value); break;
		case '5': ADD_MUL10(5,value); break;
		case '6': ADD_MUL10(6,value); break;
		case '7': ADD_MUL10(7,value); break;
		case '8': ADD_MUL10(8,value); break;
		case '9': ADD_MUL10(9,value); break;
		default:                      break;
		}
	}

	if(negative)
	{
		return -value;
	}
	else
	{
		return value;
	}
}

char* wtr_convert_i32_to_dec(wt_i32 value, char* begin, char* end)
{
	wt_bool negative = (value<0);
	wt_i32  m;
	char*   p;

	if(0==value)
	{
		begin[0] = '0';
		begin[1] = '\0';
		return begin;
	}

	if(negative)
	{
		value = -value;
	}

	p = end - 1;
	*p = '\0';

	while(value && p>begin)
	{
		MOD10(m,value);
		*--p = (char)('0' + m);
		DIV10(value);
	}

	if(negative)
	{
		*--p = '-';
	}

	return p;
}

/*================================================================< u32 >==*/

wt_u32 wtr_convert_dec_to_u32(const char* str)
{
	wt_u32 value = 0;

	while(*str)
	{
		char ch = *str++;

		switch(ch)
		{
		case '0': MUL10(value);       break;
		case '1': ADD_MUL10(1,value); break;
		case '2': ADD_MUL10(2,value); break;
		case '3': ADD_MUL10(3,value); break;
		case '4': ADD_MUL10(4,value); break;
		case '5': ADD_MUL10(5,value); break;
		case '6': ADD_MUL10(6,value); break;
		case '7': ADD_MUL10(7,value); break;
		case '8': ADD_MUL10(8,value); break;
		case '9': ADD_MUL10(9,value); break;
		default:                      break;
		}
	}

	return value;
}

char* wtr_convert_u32_to_dec(wt_u32 value, char* begin, char* end)
{
	wt_u32 m;
	char*  p;

	if(0==value)
	{
		begin[0] = '0';
		begin[1] = '\0';
		return begin;
	}

	p  = end - 1;
	*p = '\0';

	while(value && p>=begin)
	{
		MOD10(m,value);
		*--p = (char)('0' + m);
		DIV10(value);
	}

	return p;
}

/*================================================================< i64 >==*/

wt_i64 wtr_convert_dec_to_i64(const char* str)
{
	wt_i64  value    = 0;
	wt_bool negative = WT_FALSE;

	if('-'==str[0])
	{
		negative = WT_TRUE;
		++str;
	}

	while(*str)
	{
		char ch = *str++;

		switch(ch)
		{
		case '0': MUL10(value);       break;
		case '1': ADD_MUL10(1,value); break;
		case '2': ADD_MUL10(2,value); break;
		case '3': ADD_MUL10(3,value); break;
		case '4': ADD_MUL10(4,value); break;
		case '5': ADD_MUL10(5,value); break;
		case '6': ADD_MUL10(6,value); break;
		case '7': ADD_MUL10(7,value); break;
		case '8': ADD_MUL10(8,value); break;
		case '9': ADD_MUL10(9,value); break;
		default:                      break;
		}
	}

	if(negative)
	{
		return -value;
	}
	else
	{
		return value;
	}
}

char* wtr_convert_i64_to_dec(wt_i64 value, char* begin, char* end)
{
	wt_bool negative = (value<0);
	wt_i64  m;
	char*   p;

	if(0==value)
	{
		begin[0] = '0';
		begin[1] = '\0';
		return begin;
	}

	if(negative)
	{
		value = -value;
	}

	p = end - 1;
	*p = '\0';

	while(value && p>begin)
	{
		MOD10(m,value);
		*--p = (char)('0' + m);
		DIV10(value);
	}

	if(negative)
	{
		*--p = '-';
	}

	return p;
}

/*================================================================< u64 >==*/

wt_u64 wtr_convert_dec_to_u64(const char* str)
{
	wt_u64 value = 0;

	while(*str)
	{
		char ch = *str++;

		switch(ch)
		{
		case '0': MUL10(value);       break;
		case '1': ADD_MUL10(1,value); break;
		case '2': ADD_MUL10(2,value); break;
		case '3': ADD_MUL10(3,value); break;
		case '4': ADD_MUL10(4,value); break;
		case '5': ADD_MUL10(5,value); break;
		case '6': ADD_MUL10(6,value); break;
		case '7': ADD_MUL10(7,value); break;
		case '8': ADD_MUL10(8,value); break;
		case '9': ADD_MUL10(9,value); break;
		default:                      break;
		}
	}

	return value;
}

char* wtr_convert_u64_to_dec(wt_u64 value, char* begin, char* end)
{
	wt_u64 m;
	char*  p;

	if(0==value)
	{
		begin[0] = '0';
		begin[1] = '\0';
		return begin;
	}

	p  = end - 1;
	*p = '\0';

	while(value && p>=begin)
	{
		MOD10(m,value);
		*--p = (char)('0' + m);
		DIV10(value);
	}

	return p;
}

/*================================================================< f32 >==*/

#if WTRUNTIME_ENABLE_FLOAT32

wt_f32 wtr_convert_dec_to_f32(const char* str)
{
	return (wt_f32)atof(str);
}

char* wtr_convert_f32_to_dec(wt_f32 value, char* begin, char* end)
{
	/* TODO: deal with 'end' */
	((void)end);
	sprintf(begin, "%f", value);
	return begin;
}

#endif /* WTRUNTIME_ENABLE_FLOAT32 */

/*================================================================< f64 >==*/

#if WTRUNTIME_ENABLE_FLOAT64

wt_f64 wtr_convert_dec_to_f64(const char* str)
{
	return atof(str);
}

char* wtr_convert_f64_to_dec(wt_f64 value, char* begin, char* end)
{
	/* TODO: deal with 'end' */
	((void)end);
	sprintf(begin, "%lf", value);
	return begin;
}

#endif /* WTRUNTIME_ENABLE_FLOAT64 */

/*================================================================< bool >==*/

#define MATCHANY_2(_val_,_const1_,_const2_) (((_const1_)==(_val_)) || ((_const2_)==(_val_)))

wt_bool wtr_convert_dec_to_bool(const char* str)
{
	switch(*str)
	{
	case '1':
		return !str[1];
		break;

	case 'o':
	case 'O':
		return !str[2] && MATCHANY_2(str[1], 'n', 'N');
		break;

	case 'y':
	case 'Y':
		return !str[3] && MATCHANY_2(str[1], 'e', 'E') && MATCHANY_2(str[2], 's', 'S');
		break;

	case 't':
	case 'T':
		return !str[4] && MATCHANY_2(str[1], 'r', 'R') && MATCHANY_2(str[2], 'u', 'U') && MATCHANY_2(str[3], 'e', 'E');
		break;

	default:
		return WT_FALSE;
	}
}

char* wtr_convert_bool_to_dec(wt_bool value, char* begin, char* end)
{
	char* ptr = begin;

	if(end-begin<6) { return NULL; }

	if(value)
	{
		*ptr++ = 't';
		*ptr++ = 'r';
		*ptr++ = 'u';
		*ptr++ = 'e';
	}
	else
	{
		*ptr++ = 'f';
		*ptr++ = 'a';
		*ptr++ = 'l';
		*ptr++ = 's';
		*ptr++ = 'e';
	}

	*ptr = '\0';

	return begin;
}

/*==========================================================================*/
