#ifndef WTRUNTIME_CONVERT_H
#define WTRUNTIME_CONVERT_H

#include "config.h"
#include "types.h"

/*
	Instructions:
	-------------

		'from string'
			str: holds the string to convert
			returns the converted number

		'to string'
			value:      number to convert
			begin, end: defines a writeable buffer
			returns a pointer (within [begin,end) range) to the start of the result
*/

/*================================================================< i32 >==*/

/* from string */
wt_i32 wtr_convert_dec_to_i32(const char* str);

/* to string */
char* wtr_convert_i32_to_dec(wt_i32 value, char* begin, char* end);

/*================================================================< u32 >==*/

/* from string */
wt_u32 wtr_convert_dec_to_u32(const char* str);

/* to string */
char* wtr_convert_u32_to_dec(wt_u32 value, char* begin, char* end);

/*================================================================< i64 >==*/

/* from string */
wt_i64 wtr_convert_dec_to_i64(const char* str);

/* to string */
char* wtr_convert_i64_to_dec(wt_i64 value, char* begin, char* end);

/*================================================================< u64 >==*/

/* from string */
wt_u64 wtr_convert_dec_to_u64(const char* str);

/* to string */
char* wtr_convert_u64_to_dec(wt_u64 value, char* begin, char* end);

/*================================================================< f32 >==*/

#if WTRUNTIME_ENABLE_FLOAT32

/* from string */
wt_f32 wtr_convert_dec_to_f32(const char* str);

/* to string */
char* wtr_convert_f32_to_dec(wt_f32 value, char* begin, char* end);

#endif /* WTRUNTIME_ENABLE_FLOAT32 */

/*================================================================< f64 >==*/

#if WTRUNTIME_ENABLE_FLOAT64

/* from string */
wt_f64 wtr_convert_dec_to_f64(const char* str);

/* to string */
char* wtr_convert_f64_to_dec(wt_f64 value, char* begin, char* end);

#endif /* WTRUNTIME_ENABLE_FLOAT64 */

/*================================================================< bool >==*/

/* from string (the odd name is because of the macro generated codes) */
wt_bool wtr_convert_dec_to_bool(const char* str);

/* to string (the odd name is because of the macro generated codes)*/
char* wtr_convert_bool_to_dec(wt_bool value, char* begin, char* end);

/*==========================================================================*/

#endif /* WTRUNTIME_CONVERT_H */
